#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2005 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#The Notebook, and Panels.

import operator
import wx
import wx.stc
from drText import DrText

#*************************************************

def _refresh(x):
	x.Refresh()

#*******************************************************************************************************

class drNotebook(wx.Notebook):
	def __init__(self, parent, id, position, size, flags = 0):
		wx.Notebook.__init__(self, parent, id, position, size, flags)
		
		self.parent = parent
		
		self.PromptHasFocus = 0
		
		imagesize = (16,16)
		
		self.imagelist = wx.ImageList(imagesize[0], imagesize[1])
		self.images = [wx.BitmapFromImage(wx.Image(self.parent.bitmapdirectory + "/16/unmodified.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(self.parent.bitmapdirectory + "/16/modified.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(self.parent.bitmapdirectory + "/16/active unmodified.png", wx.BITMAP_TYPE_PNG)), \
		wx.BitmapFromImage(wx.Image(self.parent.bitmapdirectory + "/16/active modified.png", wx.BITMAP_TYPE_PNG))]
	
		map(self.imagelist.Add, self.images)
		
		self.AssignImageList(self.imagelist)
		
		#wxPython bug workaround, OldSelection doesn't work.
		self.oldselection = 0
		
		self.Bind(wx.EVT_LEFT_DOWN, self.OnFindFocus)
		self.Bind(wx.EVT_LEFT_UP, self.OnSelectTab)
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnPopUp)
		self.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDoubleClick)
	
	def OnFindFocus(self, event):
		self.PromptHasFocus = self.parent.txtPrompt.GetSTCFocus()
		
		event.Skip()
	
	def OnLeftDoubleClick(self, event):
		if self.parent.prefs.doubleclicktoclosetab:
			self.parent.OnClose(None)
	
	def OnPageChanged(self, event):
		if event is not None:
			i = event.GetSelection()
		else:
			i = self.GetSelection()
		l = self.GetPageCount()
		if (i < 0) or (i >= l):
			if event is not None:
				event.Skip()
			return
		if self.oldselection < l:
			self.parent.txtDocumentArray[self.oldselection].IsActive = False
			self.parent.txtDocumentArray[self.oldselection].OnModified(None)
		
		self.oldselection = i
		
		self.parent.txtDocumentArray[i].IsActive = True
		self.parent.txtDocumentArray[i].OnModified(None)
		
		if event is not None:
			event.Skip()
	
	def OnPopUp(self, event):
	
		closeallmenu = wx.Menu()				
		closeallmenu.Append(self.parent.ID_CLOSE_ALL, "Close &All Tabs"," Close All Tabs")
		closeallmenu.Append(self.parent.ID_CLOSE_ALL_OTHER_DOCUMENTS, "Close All &Other Tabs"," Close All Other Tabs")
		
		tabmenu = wx.Menu()
		tabmenu.Append(self.parent.ID_CLOSE, "&Close", "Close the file")
		tabmenu.Append(self.parent.ID_CLOSE_ALL, "Close &All Tabs")
		tabmenu.Append(self.parent.ID_CLOSE_ALL_OTHER_DOCUMENTS, "Close All &Other Tabs")
		tabmenu.AppendSeparator()
		tabmenu.Append(self.parent.ID_NEXT_DOCUMENT, "Next Tab")
		tabmenu.Append(self.parent.ID_PREVIOUS_DOCUMENT, "Previous Tab")
		tabmenu.Append(self.parent.ID_FIRST_DOCUMENT, "First Tab")
		tabmenu.Append(self.parent.ID_LAST_DOCUMENT, "Last Tab")
		tabmenu.AppendSeparator()
		tabmenu.Append(self.parent.ID_RELOAD, "&Reload File"," Reload the current file")
		tabmenu.Append(self.parent.ID_RESTORE_FROM_BACKUP, "&Restore From Backup"," Restore from the last backup made")
		tabmenu.AppendSeparator()
		tabmenu.Append(self.parent.ID_SAVE, "&Save"," Save the file")
		tabmenu.Append(self.parent.ID_SAVE_AS, "Save &As..."," Save a file as ... another file")
		
		ht = self.HitTest(event.GetPosition())[0]
		if ht > -1:
			self.SetSelection(ht)
			self.SetTab()
		
		tabmenu.Enable(self.parent.ID_RELOAD, len(self.parent.txtDocument.filename) > 0)
		tabmenu.Enable(self.parent.ID_RESTORE_FROM_BACKUP, len(self.parent.txtDocument.filename) > 0)
		self.PopupMenu(tabmenu, event.GetPosition())
		tabmenu.Destroy()

	def SetTab(self):
		selection = self.GetSelection()
		page = self.GetPage(selection)
		self.parent.setDocumentTo(selection)
		if self.PromptHasFocus and page.PromptIsVisible:
			self.parent.txtPromptArray[self.parent.docPosition].SetSTCFocus(True)
		else:
			self.parent.txtDocumentArray[self.parent.docPosition].SetSTCFocus(True)
			
	def OnSelectTab(self, event):
		selection = self.GetSelection()
		if selection is not self.parent.docPosition:
			self.SetTab()
		event.Skip()

class drSplitTextPanel(wx.Panel):
	def __init__(self, parent, grandparent, targetstc, position, index):
		docid = grandparent.txtDocument.GetId()
		wx.Panel.__init__(self, parent, docid)
		
		ID_CLOSE = grandparent.GetNewId()
		
		self.position = position
		self.index = index
		
		self.parent = parent
				
		self.grandparent = grandparent
		
		self.SetBackgroundColour(wx.Colour(0, 95, 89))
		
		if docid == targetstc.GetId():
			sv = -1
		else:
			sv = 1
		
		self.txtDoc = DrText(self, docid, grandparent, SplitView=sv)
		self.txtDoc.SetupPrefsDocument()
		self.txtDoc.SetDocPointer(targetstc.GetDocPointer())
		self.txtDoc.GotoPos(targetstc.GetCurrentPos())
		self.txtDoc.ScrollToLine(targetstc.GetCurrentLine())
		
		self.label = wx.TextCtrl(self, -1, " Viewing: " + targetstc.GetFilenameTitle(), size=(150, -1), style=wx.TE_READONLY)
		
		self.btnClose = wx.Button(self, ID_CLOSE, "x", style=wx.BU_EXACTFIT)
		
		text = self.txtDoc.GetText()
		
		#Scrolling
		linebuffer = self.txtDoc.TextWidth(wx.stc.STC_STYLE_DEFAULT, "OOO")
		lines = text.split(self.txtDoc.GetEndOfLineCharacter())
		spaces = "\t".expandtabs(self.grandparent.prefs.doctabwidth[self.txtDoc.filetype])
		line = ""
		length = 0
		for l in lines:
			if len(l) > length:
				length = len(l)
				line = l
		line = line.replace('\t', spaces)		
		self.txtDoc.SetScrollWidth(self.txtDoc.TextWidth(wx.stc.STC_STYLE_DEFAULT, line)+linebuffer)
								
		self.txtDoc.SetXOffset(0)
		#/End Scrolling
				
		self.Bind(wx.EVT_BUTTON, self.OnbtnClose, id=ID_CLOSE)		
		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
		self.Bind(wx.EVT_SIZE, self.OnSize)

	def OnbtnClose(self, event):
		self.grandparent.currentpage.ShowPanel(self.position, self.index, False)
										
	def OnKeyDown(self, event):
		self.grandparent.RunShortcuts(event)

		event.Skip()
	
	def OnSize(self, event):
		x, y = self.GetSizeTuple()
		lX, lY = self.label.GetSizeTuple()
		bX, bY = self.btnClose.GetSizeTuple()
		pX, pY = self.GetPositionTuple()
		self.txtDoc.Move((pX,pY+lY))
		self.label.SetSize((x-bX, lY))
		self.btnClose.Move((x-bX, pY))
		self.txtDoc.SetSize((x, y-lY))
		if (x <= 0) or (y <= 0):
			self.Destroy()

class drSashWindow(wx.SashWindow):
	def __init__(self, parent, id, pos, size, style):
		wx.SashWindow.__init__(self, parent, id, pos, size, style)
				
		self.grandparent = parent.grandparent
		
		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
										
	def OnKeyDown(self, event):
		self.grandparent.RunShortcuts(event)

		event.Skip()

class drDocPanel(wx.Panel):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		
		self.grandparent = parent.grandparent
		
		width, height = self.GetSizeTuple()
		
		#Variables to Keep Track of what is being used.
			
		self.ID_LEFT = 6300 + id
		self.ID_RIGHT = 6400 + id
		self.ID_TOP = 6500 + id
		self.ID_BOTTOM = 6600 + id
				
		self.centertuple = (width, height)
		
		self.center = drSashWindow(self, id, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.lefttuples = []
		self.righttuples = []
		self.toptuples = []
		self.bottomtuples = []

		self.leftsizes = []
		self.rightsizes = []
		self.bottomsizes = []
		self.topsizes = []
		
		self.leftsizestores = []
		self.rightsizestores = []
		self.bottomsizestores = []
		self.topsizestores = []
		
		self.LeftSashWindows = []
		self.RightSashWindows = []
		self.TopSashWindows = []
		self.BottomSashWindows = []
						
		self.LeftIsVisible = []
		self.RightIsVisible = []
		self.TopIsVisible = []
		self.BottomIsVisible = []
		
		self.oldwidth, self.oldheight = 0, 0
		
		self.Bind(wx.EVT_SIZE, self.OnSize)

	def CalculateSizes(self, width, height, left, right, top, bottom):
		lefttotal, lefttupletotal = 0, 0
		righttotal, righttupletotal = 0, 0
		toptotal, toptupletotal = 0, 0
		bottomtotal, bottomtupletotal = 0, 0		

		def _PercentWidth_Height(x):
			return (((width * x) / 100), height)
		
		def _Width_PercentHeight(x):
			return (width, ((height * x) / 100))
		
		def _PercentWidth(x):
			return (width * x) / 100
		
		def _PercentHeight(x):
			return (height * x) / 100
		
		#Left
		if left > 0:
			lefttotal = self.Sum(self.leftsizes)
			self.lefttuples = map(_PercentWidth_Height, self.leftsizes)
			lefttupletotal = self.Sum(map(_PercentWidth, self.leftsizes))
			
		#Right
		if right > 0:
			righttotal = self.Sum(self.rightsizes)	
			self.righttuples = map(_PercentWidth_Height, self.rightsizes)
			righttupletotal = self.Sum(map(_PercentWidth, self.rightsizes))			
				
		#Top
		if top > 0:
			toptotal = self.Sum(self.topsizes)
			self.toptuples = map(_Width_PercentHeight, self.topsizes)
			toptupletotal = self.Sum(map(_PercentHeight, self.topsizes))
		
		#Bottom
		if bottom > 0:
			bottomtotal = self.Sum(self.bottomsizes)
			self.bottomtuples = map(_Width_PercentHeight, self.bottomsizes)
			bottomtupletotal = self.Sum(map(_PercentHeight, self.bottomsizes))			

		fixleft = 0
		fixright = 0
		fixtop = 0
		fixbottom = 0
		
		if width > 0:
			if lefttupletotal >= width:
				fixleft = 1
			if righttupletotal >= width:
				fixright = 1
			if lefttupletotal + righttupletotal >= width:
				fixleft = 1			
				fixright = 1

		if height > 0:
			if toptupletotal >= height:
				fixtop = 1
			if bottomtupletotal >= height:
				fixbottom = 1
			if toptupletotal + bottomtupletotal >= height:
				fixtop = 1			
				fixbottom = 1

		if fixleft:
			a = list(self.leftsizes)
			self.leftsizes = []
			for s in a:
				self.leftsizes.append(int(s / 2))

		if fixright:
			a = list(self.rightsizes)
			self.rightsizes = []
			for s in a:
				self.rightsizes.append(int(s / 2))			

		if fixtop:
			a = list(self.topsizes)
			self.topsizes = []
			for s in a:
				self.topsizes.append(int(s / 2))
			
		if fixbottom:
			a = list(self.bottomsizes)
			self.bottomsizes = []
			for s in a:
				self.bottomsizes.append(int(s / 2))

		if fixleft or fixright or fixtop or fixbottom:
			return self.CalculateSizes(width, height, left, right, top, bottom)

		return 	(lefttotal, lefttupletotal, righttotal, righttupletotal, toptotal, toptupletotal, bottomtotal, bottomtupletotal)
	
	def ClosePanel(self, Position, Index):
		if Position == 0:			
			self.leftsizes.pop(Index)
			self.leftsizestores.pop(Index)
			self.lefttuples.pop(Index)
			self.LeftSashWindows[Index].Destroy()
			self.LeftSashWindows.pop(Index)
			self.LeftIsVisible.pop(Index)
		elif Position == 1:
			self.rightsizes.pop(Index)
			self.rightsizestores.pop(Index)
			self.righttuples.pop(Index)
			self.RightSashWindows[Index].Destroy()
			self.RightSashWindows.pop(Index)
			self.RightIsVisible.pop(Index)
		elif Position == 2:
			self.topsizes.pop(Index)
			self.topsizestores.pop(Index)
			self.toptuples.pop(Index)			
			self.TopSashWindows[Index].Destroy()
			self.TopSashWindows.pop(Index)
			self.TopIsVisible.pop(Index)
		elif Position == 3:
			self.bottomsizes.pop(Index)
			self.bottomsizestores.pop(Index)
			self.bottomtuples.pop(Index)
			self.BottomSashWindows[Index].Destroy()
			self.BottomSashWindows.pop(Index)
			self.BottomIsVisible.pop(Index)
		self.OnSize(None)
		self.grandparent.txtDocument.SetFocus()	
		
	def GetTargetSashWindow(self, Position=1):
		if Position == 0:
			i = len(self.LeftSashWindows)
			left = drSashWindow(self, self.ID_LEFT+i, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)
			left.SetSashVisible(wx.SASH_RIGHT, True)
			left.SetSashBorder(wx.SASH_RIGHT, True)
			
			self.LeftSashWindows.append(left)
			self.LeftIsVisible.append(True)
			
			self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_LEFT+i)
			
			target = self.LeftSashWindows[i]
			self.leftsizes.append(self.grandparent.prefs.sidepanelleftsize)
			self.leftsizestores.append(self.grandparent.prefs.sidepanelleftsize)
			self.lefttuples.append((0, 0))
		elif Position == 1:
			i = len(self.RightSashWindows)
			right = drSashWindow(self, self.ID_RIGHT+i, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)
			right.SetSashVisible(wx.SASH_LEFT, True)
			right.SetSashBorder(wx.SASH_LEFT, True)
			
			self.RightSashWindows.append(right)
			self.RightIsVisible.append(True)
			
			self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_RIGHT+i)
			
			target = self.RightSashWindows[i]
			self.rightsizes.append(self.grandparent.prefs.sidepanelrightsize)
			self.rightsizestores.append(self.grandparent.prefs.sidepanelrightsize)
			self.righttuples.append((0, 0))
		elif Position == 2:
			i = len(self.TopSashWindows)
			top = drSashWindow(self, self.ID_TOP+i, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)
			top.SetSashVisible(wx.SASH_BOTTOM, True)
			top.SetSashBorder(wx.SASH_BOTTOM, True)
			
			self.TopSashWindows.append(top)
			self.TopIsVisible.append(True)
			
			self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_TOP+i)
			
			target = self.TopSashWindows[i]
			self.topsizes.append(self.grandparent.prefs.sidepaneltopsize)
			self.topsizestores.append(self.grandparent.prefs.sidepaneltopsize)
			self.toptuples.append((0, 0))
		elif Position == 3:
			i = len(self.BottomSashWindows)
			bottom = drSashWindow(self, self.ID_BOTTOM+i, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)
			bottom.SetSashVisible(wx.SASH_TOP, True)
			bottom.SetSashBorder(wx.SASH_TOP, True)
			
			self.BottomSashWindows.append(bottom)
			self.BottomIsVisible.append(True)
			
			self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_BOTTOM+i)
			
			target = self.BottomSashWindows[i]
			self.bottomsizes.append(self.grandparent.prefs.sidepanelbottomsize)
			self.bottomsizestores.append(self.grandparent.prefs.sidepanelbottomsize)
			self.bottomtuples.append((0, 0))
		return target, i

	def IsVisible(self, Position, Index):
		if Position == 0:
			return self.LeftIsVisible[Index]
		elif Position == 1:
			return self.RightIsVisible[Index]
		elif Position == 2:
			return self.TopIsVisible[Index]
		elif Position == 3:
			return self.BottomIsVisible[Index]		

	def MoveToH(self, sasharray, tuplearray, w):
		l = len(tuplearray)
		x = 0
		while x < l:
			pos = self.SumTuple(tuplearray[:x], 0)
			sasharray[x].Move((pos+w, 0))
			x = x + 1

	def MoveToV(self, sasharray, tuplearray, w, h):
		x = 0
		l = len(tuplearray)
		while x < l:
			pos = self.SumTuple(tuplearray[:x], 1)
			sasharray[x].Move((w, pos + h))
			x = x + 1
		
	def OnSashDrag(self, event):
		evtheight = event.GetDragRect().height
		evtwidth = event.GetDragRect().width
		width, height = self.GetSizeTuple()
		if (evtwidth < 0):
			evtwidth = 0
		elif (evtwidth > width):
			evtwidth = width
		if event.GetDragStatus() == wx.SASH_STATUS_OUT_OF_RANGE:
			if (evtheight < height):
				evtheight = 0
			else:
				evtheight = height
		elif evtheight > height:
			evtheight = height

		#Edge Drag
		e = event.GetId()
		if (e >= self.ID_LEFT) and (e < self.ID_RIGHT):
			i = e - self.ID_LEFT
			if self.lefttuples[i][0] == evtwidth:
				evtwidth = 0
				self.LeftIsVisible[i] = False
			self.lefttuples[i] = (evtwidth, height)			
			self.centertuple = (width-evtwidth, self.centertuple[1])
			self.leftsizes[i] = (evtwidth*100) / width
		elif (e >= self.ID_RIGHT) and (e < self.ID_TOP):
			i = e - self.ID_RIGHT
			if self.righttuples[i][0] == evtwidth:
				evtwidth = 0
				self.RightIsVisible[i] = False
			self.righttuples[i] = (evtwidth, height)
			self.centertuple = (width-evtwidth, self.centertuple[1])
			self.rightsizes[i] = (evtwidth*100) / width
		elif (e >= self.ID_TOP) and (e < self.ID_BOTTOM):
			i = e - self.ID_TOP
			if self.toptuples[i][1] == evtheight:
				evtheight = 0
				self.TopIsVisible[i] = False
			self.toptuples[i] = (self.toptuples[i][0], evtheight)
			self.centertuple = (self.centertuple[0], self.centertuple[1]-evtheight)
			self.topsizes[i] = (evtheight*100) / height
		elif e >= self.ID_BOTTOM:
			i = e - self.ID_BOTTOM
			if self.bottomtuples[i][1] == evtheight:
				evtheight = 0
				self.BottomIsVisible[i] = False
			self.bottomtuples[i] = (self.bottomtuples[i][0], evtheight)
			self.centertuple = (self.centertuple[0], self.centertuple[1]-evtheight)
			self.bottomsizes[i] = (evtheight*100) / height
			
		self.OnSize(None)	

	def OnSize(self, event):
		width, height = self.GetSizeTuple()
		if (event is not None) and (width == self.oldwidth) and (height == self.oldheight):
			return
		self.oldwidth, self.oldheight = width, height
				
		lleft, lright, ltop, lbottom = len(self.leftsizes), len(self.rightsizes), len(self.topsizes), len(self.bottomsizes)

		lefttotal, lefttupletotal, righttotal, righttupletotal, toptotal, toptupletotal, bottomtotal, bottomtupletotal = \
		self.CalculateSizes(width, height, lleft, lright, ltop, lbottom)

		#Width
		w = (width * (100 - lefttotal - righttotal)) / 100			
		
		#Height	
		h = (height * (100 - toptotal - bottomtotal)) / 100


		#Panels
		if lleft > 0:
			self.SetSizeFor(self.LeftSashWindows, self.lefttuples)
			self.MoveToH(self.LeftSashWindows, self.lefttuples, 0)
			map(_refresh, self.LeftSashWindows)			
		
		if lright > 0:
			self.SetSizeFor(self.RightSashWindows, self.righttuples)
			self.MoveToH(self.RightSashWindows, self.righttuples, w + lefttupletotal)
			map(_refresh, self.RightSashWindows)
			
		if ltop > 0:
			self.SetSizeFor(self.TopSashWindows, self.toptuples)
			self.MoveToV(self.TopSashWindows, self.toptuples, lefttupletotal, 0)
			map(_refresh, self.TopSashWindows)			
			
		if lbottom > 0:
			self.SetSizeFor(self.BottomSashWindows, self.bottomtuples)
			self.MoveToV(self.BottomSashWindows, self.bottomtuples, lefttupletotal, h + toptupletotal)
			map(_refresh, self.BottomSashWindows)
		
		#Tuples
		self.centertuple = (w, h)		

		#Set Size then Move
		self.center.SetSize(self.centertuple)

		self.center.Move((lefttupletotal,toptupletotal))				

	def SetSizeFor(self, sasharray, tuplearray):
		x = 0
		l = len(tuplearray)
		while x < l:
			sasharray[x].SetSize(tuplearray[x])
			x = x + 1
			
	def ShowPanel(self, Position, Index, Show):
		if Position == 0:
			self.LeftIsVisible[Index] = Show
			if Show:
				self.leftsizes[Index] = self.leftsizestores[Index]
			else:
				self.leftsizestores[Index] = self.leftsizes[Index]
				self.leftsizes[Index] = 0
		elif Position == 1:
			self.RightIsVisible[Index] = Show
			if Show:
				self.rightsizes[Index] = self.rightsizestores[Index]
			else:
				self.rightsizestores[Index] = self.rightsizes[Index]
				self.rightsizes[Index] = 0	
		elif Position == 2:
			self.TopIsVisible[Index] = Show
			if Show:
				self.topsizes[Index] = self.topsizestores[Index]
			else:
				self.topsizestores[Index] = self.topsizes[Index]
				self.topsizes[Index] = 0
		elif Position == 3:
			self.BottomIsVisible[Index] = Show
			if Show:
				self.bottomsizes[Index] = self.bottomsizestores[Index]
			else:
				self.bottomsizestores[Index] = self.bottomsizes[Index]
				self.bottomsizes[Index] = 0
		self.OnSize(None)		
	
	def Sum(self, seq):
		if len(seq) > 0:
			return reduce(operator.add, seq)
		return 0
			
	def SumTuple(self, tuplearray, i):
		sum = 0
		for t in tuplearray:
			sum = sum + t[i]
		return sum
	
	def TogglePanel(self, Position, Index):
		if Position == 0:
			self.LeftIsVisible[Index] = not self.LeftIsVisible[Index]
			if self.LeftIsVisible[Index]:
				self.leftsizes[Index] = self.leftsizestores[Index]
			else:
				self.leftsizestores[Index] = self.leftsizes[Index]
				self.leftsizes[Index] = 0
		elif Position == 1:
			self.RightIsVisible[Index] = not self.RightIsVisible[Index]
			if self.RightIsVisible[Index]:
				self.rightsizes[Index] = self.rightsizestores[Index]
			else:
				self.rightsizestores[Index] = self.rightsizes[Index]
				self.rightsizes[Index] = 0	
		elif Position == 2:
			self.TopIsVisible[Index] = not self.TopIsVisible[Index]
			if self.TopIsVisible[Index]:
				self.topsizes[Index] = self.topsizestores[Index]
			else:
				self.topsizestores[Index] = self.topsizes[Index]
				self.topsizes[Index] = 0
		elif Position == 3:
			self.BottomIsVisible[Index] = not self.BottomIsVisible[Index]
			if self.BottomIsVisible[Index]:
				self.bottomsizes[Index] = self.bottomsizestores[Index]
			else:
				self.bottomsizestores[Index] = self.bottomsizes[Index]
				self.bottomsizes[Index] = 0
		self.OnSize(None)
		
		
class drPanel(wx.Panel):
	def __init__(self, parent, id):
		wx.Panel.__init__(self, parent, id)
		
		self.grandparent = parent.GetParent()
		
		width, height = self.GetSizeTuple()
		
		#Variables to Keep Track of what is being used.
			
		self.ID_DOCUMENT = 6001 + id
		self.ID_PROMPT = 6002 + id
		
		self.PromptIsVisible = self.grandparent.prefs.promptisvisible
				
		self.documenttuple = (width, height)	
		self.prompttuple = (0, 0)

		self.promptsize = self.grandparent.prefs.promptsize

		self.prompt = drSashWindow(self, self.ID_PROMPT, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.prompt.SetSashVisible(wx.SASH_TOP, True)
		self.prompt.SetSashBorder(wx.SASH_TOP, True)

		self.document = drSashWindow(self, self.ID_DOCUMENT, wx.DefaultPosition, wx.DefaultSize, wx.SW_3D)

		self.documentpanel = drDocPanel(self.document, id)

		self.document.SetSashVisible(wx.SASH_BOTTOM, True)
		self.document.SetSashBorder(wx.SASH_BOTTOM, True)

		self.oldwidth, self.oldheight = 0, 0

		self.Bind(wx.EVT_SIZE, self.OnSize)
		self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_DOCUMENT)
		self.Bind(wx.EVT_SASH_DRAGGED, self.OnSashDrag, id=self.ID_PROMPT)
					
	def ClosePanel(self, Position, Index):
		self.documentpanel.ClosePanel(Position, Index)
	
	def GetTargetSashWindow(self, Position=1):
		return self.documentpanel.GetTargetSashWindow(Position)	
	
	def IsVisible(self, Position, Index):
		return self.documentpanel.IsVisible(Position, Index)
		
	def OnSashDrag(self, event):
		evtheight = event.GetDragRect().height
		evtwidth = event.GetDragRect().width
		width, height = self.GetSizeTuple()
		if (evtwidth < 0):
			evtwidth = 0
		elif (evtwidth > width):
			evtwidth = width
		if event.GetDragStatus() == wx.SASH_STATUS_OUT_OF_RANGE:
			if (not self.PromptIsVisible) or (evtheight < height):
				evtheight = 0
			else:
				evtheight = height
		elif evtheight > height:
			evtheight = height

		oldsize = self.promptsize

		#Edge Drag
		e = event.GetId()
		edge = event.GetEdge()
		if e == self.ID_DOCUMENT:
			if edge == wx.SASH_BOTTOM:
				self.promptsize = ((height*100) - (evtheight*100)) / height
				self.documenttuple = (self.documenttuple[0], evtheight)
				self.prompttuple = (self.prompttuple[0], height-evtheight)
		elif e == self.ID_PROMPT:
			self.promptsize = ((evtheight*100) / height)
			
		#Prompt Is Visible
		if self.promptsize == 0:
			self.promptsize = oldsize
			self.PromptIsVisible = False
		elif not self.PromptIsVisible and self.prompttuple[1] > 0:
			self.PromptIsVisible = True
					
		self.OnSize(None)		

	def OnSize(self, event):
		width, height = self.GetSizeTuple()		
		if (event is not None) and (width == self.oldwidth) and (height == self.oldheight):
			return
		self.oldwidth, self.oldheight = width, height
		
		if self.PromptIsVisible:
			heightDocument = (height * (100 - self.promptsize)) / 100
			heightPrompt = (height * self.promptsize) / 100
		else:
			heightDocument = height
			heightPrompt = 0
		
		#Tuples
		self.documenttuple = (width, heightDocument)
		self.prompttuple = (width, heightPrompt)	

		#Set Size then Move
		self.document.SetSize(self.documenttuple)
		self.documentpanel.SetSize(self.documenttuple)
		self.prompt.SetSize(self.prompttuple)

		self.document.Move((0,0))
		self.prompt.Move((0, heightDocument))
		
		#Size the document panel.
		self.documentpanel.OnSize(None)	
		
	def ShowPanel(self, Position, Index, Show):
		self.documentpanel.ShowPanel(Position, Index, Show)
		
	def TogglePanel(self, Position, Index):
		self.documentpanel.TogglePanel(Position, Index)